@@ -1,23 +1,28 @@
 """Standalone Authenticator."""
 import collections
+import errno
 import logging
 import socket
-# https://github.com/python/typeshed/blob/master/stdlib/2and3/socket.pyi
-from socket import errno as socket_errors  # type: ignore
+from typing import Any
+from typing import Callable
+from typing import DefaultDict
+from typing import Dict
+from typing import Iterable
+from typing import List
+from typing import Mapping
+from typing import Set
+from typing import Tuple
+from typing import Type
+from typing import TYPE_CHECKING
 
-import OpenSSL  # pylint: disable=unused-import
-import zope.interface
+from OpenSSL import crypto
 
 from acme import challenges
 from acme import standalone as acme_standalone
-from acme.magic_typing import DefaultDict
-from acme.magic_typing import Dict
-from acme.magic_typing import Set
-from acme.magic_typing import Tuple
-from acme.magic_typing import TYPE_CHECKING
 from certbot import achallenges
 from certbot import errors
 from certbot import interfaces
+from certbot.display import util as display_util
 from certbot.plugins import common
 
 logger = logging.getLogger(__name__)
@@ -25,9 +30,10 @@ logger = logging.getLogger(__name__)
 if TYPE_CHECKING:
     ServedType = DefaultDict[
         acme_standalone.BaseDualNetworkedServers,
-        Set[achallenges.KeyAuthorizationAnnotatedChallenge]
+        Set[achallenges.AnnotatedChallenge]
     ]
 
+
 class ServerManager:
     """Standalone servers manager.
 
@@ -41,12 +47,15 @@ class ServerManager:
     will serve the same URLs!
 
     """
-    def __init__(self, certs, http_01_resources):
-        self._instances = {}  # type: Dict[int, acme_standalone.BaseDualNetworkedServers]
+    def __init__(self, certs: Mapping[bytes, Tuple[crypto.PKey, crypto.X509]],
+                 http_01_resources: Set[acme_standalone.HTTP01RequestHandler.HTTP01Resource]
+                 ) -> None:
+        self._instances: Dict[int, acme_standalone.HTTP01DualNetworkedServers] = {}
         self.certs = certs
         self.http_01_resources = http_01_resources
 
-    def run(self, port, challenge_type, listenaddr=""):
+    def run(self, port: int, challenge_type: Type[challenges.Challenge],
+            listenaddr: str = "") -> acme_standalone.HTTP01DualNetworkedServers:
         """Run ACME server on specified ``port``.
 
         This method is idempotent, i.e. all calls with the same pair of
@@ -80,7 +89,7 @@ class ServerManager:
         self._instances[real_port] = servers
         return servers
 
-    def stop(self, port):
+    def stop(self, port: int) -> None:
         """Stop ACME server running on the specified ``port``.
 
         :param int port:
@@ -93,7 +102,7 @@ class ServerManager:
         instance.shutdown_and_server_close()
         del self._instances[port]
 
-    def running(self):
+    def running(self) -> Dict[int, acme_standalone.HTTP01DualNetworkedServers]:
         """Return all running instances.
 
         Once the server is stopped using `stop`, it will not be
@@ -106,9 +115,7 @@ class ServerManager:
         return self._instances.copy()
 
 
-@zope.interface.implementer(interfaces.IAuthenticator)
-@zope.interface.provider(interfaces.IPluginFactory)
-class Authenticator(common.Plugin):
+class Authenticator(common.Plugin, interfaces.Authenticator):
     """Standalone Authenticator.
 
     This authenticator creates its own ephemeral TCP listener on the
@@ -117,56 +124,62 @@ class Authenticator(common.Plugin):
     rely on any existing server program.
     """
 
-    description = "Spin up a temporary webserver"
+    description = """Runs an HTTP server locally which serves the necessary validation files \
+under the /.well-known/acme-challenge/ request path. Suitable if there is no HTTP server already \
+running. HTTP challenge only (wildcards not supported)."""
 
-    def __init__(self, *args, **kwargs):
-        super(Authenticator, self).__init__(*args, **kwargs)
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
 
-        self.served = collections.defaultdict(set)  # type: ServedType
+        self.served: ServedType = collections.defaultdict(set)
 
         # Stuff below is shared across threads (i.e. servers read
         # values, main thread writes). Due to the nature of CPython's
         # GIL, the operations are safe, c.f.
         # https://docs.python.org/2/faq/library.html#what-kinds-of-global-value-mutation-are-thread-safe
-        self.certs = {}  # type: Dict[bytes, Tuple[OpenSSL.crypto.PKey, OpenSSL.crypto.X509]]
-        self.http_01_resources = set() \
-        # type: Set[acme_standalone.HTTP01RequestHandler.HTTP01Resource]
+        self.certs: Mapping[bytes, Tuple[crypto.PKey, crypto.X509]] = {}
+        self.http_01_resources: Set[acme_standalone.HTTP01RequestHandler.HTTP01Resource] = set()
 
         self.servers = ServerManager(self.certs, self.http_01_resources)
 
     @classmethod
-    def add_parser_arguments(cls, add):
+    def add_parser_arguments(cls, add: Callable[..., None]) -> None:
         pass  # No additional argument for the standalone plugin parser
 
-    def more_info(self):  # pylint: disable=missing-function-docstring
+    def more_info(self) -> str:  # pylint: disable=missing-function-docstring
         return("This authenticator creates its own ephemeral TCP listener "
                "on the necessary port in order to respond to incoming "
                "http-01 challenges from the certificate authority. Therefore, "
                "it does not rely on any existing server program.")
 
-    def prepare(self):  # pylint: disable=missing-function-docstring
+    def prepare(self) -> None:  # pylint: disable=missing-function-docstring
         pass
 
-    def get_chall_pref(self, domain):
+    def get_chall_pref(self, domain: str) -> Iterable[Type[challenges.Challenge]]:
         # pylint: disable=unused-argument,missing-function-docstring
         return [challenges.HTTP01]
 
-    def perform(self, achalls):  # pylint: disable=missing-function-docstring
+    def perform(self, achalls: Iterable[achallenges.AnnotatedChallenge]
+                ) -> List[challenges.ChallengeResponse]:  # pylint: disable=missing-function-docstring
         return [self._try_perform_single(achall) for achall in achalls]
 
-    def _try_perform_single(self, achall):
+    def _try_perform_single(self,
+                            achall: achallenges.AnnotatedChallenge) -> challenges.ChallengeResponse:
         while True:
             try:
                 return self._perform_single(achall)
             except errors.StandaloneBindError as error:
                 _handle_perform_error(error)
 
-    def _perform_single(self, achall):
+    def _perform_single(self,
+                        achall: achallenges.AnnotatedChallenge) -> challenges.ChallengeResponse:
         servers, response = self._perform_http_01(achall)
         self.served[servers].add(achall)
         return response
 
-    def _perform_http_01(self, achall):
+    def _perform_http_01(self, achall: achallenges.AnnotatedChallenge
+                         ) -> Tuple[acme_standalone.HTTP01DualNetworkedServers,
+                                    challenges.ChallengeResponse]:
         port = self.config.http01_port
         addr = self.config.http01_address
         servers = self.servers.run(port, challenges.HTTP01, listenaddr=addr)
@@ -176,7 +189,7 @@ class Authenticator(common.Plugin):
         self.http_01_resources.add(resource)
         return servers, response
 
-    def cleanup(self, achalls):  # pylint: disable=missing-function-docstring
+    def cleanup(self, achalls: Iterable[achallenges.AnnotatedChallenge]) -> None:  # pylint: disable=missing-function-docstring
         # reduce self.served and close servers if no challenges are served
         for unused_servers, server_achalls in self.served.items():
             for achall in achalls:
@@ -186,23 +199,29 @@ class Authenticator(common.Plugin):
             if not self.served[servers]:
                 self.servers.stop(port)
 
+    def auth_hint(self, failed_achalls: List[achallenges.AnnotatedChallenge]) -> str:
+        port, addr = self.config.http01_port, self.config.http01_address
+        neat_addr = f"{addr}:{port}" if addr else f"port {port}"
+        return ("The Certificate Authority failed to download the challenge files from "
+                f"the temporary standalone webserver started by Certbot on {neat_addr}. "
+                "Ensure that the listed domains point to this machine and that it can "
+                "accept inbound connections from the internet.")
 
-def _handle_perform_error(error):
-    if error.socket_error.errno == socket_errors.EACCES:
+
+def _handle_perform_error(error: errors.StandaloneBindError) -> None:
+    if error.socket_error.errno == errno.EACCES:
         raise errors.PluginError(
             "Could not bind TCP port {0} because you don't have "
             "the appropriate permissions (for example, you "
             "aren't running this program as "
             "root).".format(error.port))
-    if error.socket_error.errno == socket_errors.EADDRINUSE:
-        display = zope.component.getUtility(interfaces.IDisplay)
+    if error.socket_error.errno == errno.EADDRINUSE:
         msg = (
             "Could not bind TCP port {0} because it is already in "
             "use by another process on this system (such as a web "
             "server). Please stop the program in question and "
             "then try again.".format(error.port))
-        should_retry = display.yesno(msg, "Retry",
-                                     "Cancel", default=False)
+        should_retry = display_util.yesno(msg, "Retry", "Cancel", default=False)
         if not should_retry:
             raise errors.PluginError(msg)
     else:
